/* $Id: cpl_detector-test.c,v 1.18 2008/12/15 14:13:50 cizzo Exp $
 *
 * This file is part of the ESO Common Pipeline Library
 * Copyright (C) 2001-2008 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: cizzo $
 * $Date: 2008/12/15 14:13:50 $
 * $Revision: 1.18 $
 * $Name: cpl-5_0_0 $
 */

/*-----------------------------------------------------------------------------
                                   Includes
 -----------------------------------------------------------------------------*/

#include <math.h>

#include <cpl_image_gen.h>
#include <cpl_image_bpm.h>
#include <cpl_image_io.h>
#include <cpl_test.h>
#include <cpl_memory.h>

#include "cpl_detector.h"

/*-----------------------------------------------------------------------------
                                  Define
 -----------------------------------------------------------------------------*/

#define IMAGESZ         10
#define IMAGESZ2        100

/*-----------------------------------------------------------------------------
                                  Main
 -----------------------------------------------------------------------------*/
int main(void)
{
    cpl_image * im;
    cpl_image * copy;
    cpl_image * load;
    const int   nbad = 10;
    /* the bad pixels positions */
    const int   bad_pos_x[] = {4, 5, 6, 4, 5, 6, 4, 5, 6, 8};
    const int   bad_pos_y[] = {5, 5, 5, 6, 6, 6, 7, 7, 7, 9};
    int         pos;
    double      noise, stdev;
    /* Two concentric circles in the image center */
    double zone[] = {0.5*IMAGESZ2, 0.5*IMAGESZ2, IMAGESZ2/10.0, IMAGESZ2/5.0};
    int         i;


    cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_WARNING);

    /* Insert tests below */

    /* Test 0: Test with nbad pixels */
    cpl_test_eq(nbad, sizeof(bad_pos_x)/sizeof(bad_pos_x[0]));
    cpl_test_eq(nbad, sizeof(bad_pos_y)/sizeof(bad_pos_y[0]));

    /* Create the test image */
    im = cpl_image_new(IMAGESZ, IMAGESZ, CPL_TYPE_DOUBLE);
    cpl_test_eq(cpl_image_fill_noise_uniform(im, 10, 20), CPL_ERROR_NONE);
    cpl_test_error(CPL_ERROR_NONE);

    /* Test 1: cpl_detector_interpolate_rejected() may not modify a bad-pixel
       free image */
    copy = cpl_image_duplicate(im);
    cpl_test_eq(cpl_detector_interpolate_rejected(copy), CPL_ERROR_NONE);
    cpl_test_error(CPL_ERROR_NONE);

    cpl_test_zero(cpl_image_count_rejected(copy));

    cpl_test_eq(cpl_image_subtract(copy, im), CPL_ERROR_NONE);
    cpl_test_error(CPL_ERROR_NONE);

    cpl_test_abs(cpl_image_get_absflux(copy), 0.0, DBL_EPSILON);
    cpl_image_delete(copy);

    /* Set the bad pixels values */
    for (i=0 ; i<nbad ; i++) {
        pos = bad_pos_x[i]-1 + (bad_pos_y[i]-1) * IMAGESZ;
	cpl_image_set(im, bad_pos_x[i], bad_pos_y[i], FLT_MAX);
        cpl_image_reject(im, bad_pos_x[i], bad_pos_y[i]);
    }

    cpl_test_eq(cpl_image_count_rejected(im), nbad);

    stdev = cpl_image_get_stdev(im);

    copy = cpl_image_cast(im, CPL_TYPE_INT);

    /* Test 2: cpl_detector_interpolate_rejected() on some bad pixels */
    cpl_test_eq(cpl_detector_interpolate_rejected(im), CPL_ERROR_NONE);
    cpl_test_error(CPL_ERROR_NONE);

    cpl_test_zero(cpl_image_count_rejected(im));

    cpl_test_leq(cpl_image_get_stdev(im), stdev);

    /* In absence of an actual unit test for the cleaning, a regression
       test may be used to ensure that no unintended changes occur */
    load = cpl_image_load("regression.fits", CPL_TYPE_DOUBLE, 0, 0);

    if (load == NULL) {
      cpl_test_error(CPL_ERROR_FILE_IO);
      /*
       Enable this call before making changes to 
       cpl_detector_interpolate_rejected()
       to enable a regression test of your changes.
      */
#if 0
       cpl_image_save(im, "regression.fits", CPL_BPP_IEEE_DOUBLE, NULL,
                      CPL_IO_CREATE);
#endif

    } else {
      cpl_test_error(CPL_ERROR_NONE);
      cpl_test_image_abs(load, im, DBL_EPSILON);

      cpl_image_delete(load);
    }

    cpl_image_delete(im);

    /* Create an other test image */
    im = cpl_image_new(IMAGESZ2, IMAGESZ2, CPL_TYPE_DOUBLE);
    cpl_image_fill_noise_uniform(im, 10, 20);

    /* Test 3: the RON computation */
    cpl_test_eq(cpl_flux_get_noise_window(im, NULL, -1, -1, &noise, NULL),
		CPL_ERROR_NONE);
    cpl_test_error(CPL_ERROR_NONE);

    /* Test 4: the RON computation - on a ring */
    cpl_test_eq(cpl_flux_get_noise_ring(im, zone, -1, -1, &noise, NULL),
		CPL_ERROR_NONE);
    cpl_test_error(CPL_ERROR_NONE);

    cpl_image_delete(im);

    /* Test 5: unsupported input */
    cpl_test_eq(cpl_detector_interpolate_rejected(copy),
		CPL_ERROR_INVALID_TYPE);
    cpl_test_error(CPL_ERROR_INVALID_TYPE);

    cpl_image_delete(copy);

    /* Test 6: NULL input */
    cpl_test_eq(cpl_detector_interpolate_rejected(NULL), CPL_ERROR_NULL_INPUT);
    cpl_test_error(CPL_ERROR_NULL_INPUT);

    cpl_test_eq(cpl_flux_get_noise_window(NULL, NULL, -1, -1, &noise, NULL),
		CPL_ERROR_NULL_INPUT);
    cpl_test_error(CPL_ERROR_NULL_INPUT);

    cpl_test_eq(cpl_flux_get_noise_ring(NULL, zone, -1, -1, &noise, NULL),
		CPL_ERROR_NULL_INPUT);
    cpl_test_error(CPL_ERROR_NULL_INPUT);

    cpl_test_eq(cpl_flux_get_noise_ring(im, NULL, -1, -1, &noise, NULL),
		CPL_ERROR_NULL_INPUT);
    cpl_test_error(CPL_ERROR_NULL_INPUT);

    cpl_test_eq(cpl_flux_get_noise_ring(im, zone, -1, -1, NULL, NULL),
		CPL_ERROR_NULL_INPUT);
    cpl_test_error(CPL_ERROR_NULL_INPUT);

    /* Test 7: An empty ring */
    zone[3] = zone[2];
    cpl_test_eq(cpl_flux_get_noise_ring(im, zone, -1, -1, &noise, NULL),
		CPL_ERROR_ILLEGAL_INPUT);
    cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);

    return cpl_test_end(0);
}
